@@ -6,13 +6,14 @@ use feature 'switch';
 use strict;
 use warnings;
 
-our $VERSION = 1.28;
+our $VERSION = '1.34';
 
 # vim: ts=4:sw=4:et:ai:sts=4
 #
 # KGB - an IRC bot helping collaboration
-# Copyright © 2008 Martín Ferrari
+# Copyright © 2008 Martina Ferrari
 # Copyright © 2009,2010,2011,2012,2013 Damyan Ivanov
+# Copyright © 2018 Colin Finck <colin@reactos.org>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -231,6 +232,11 @@ The login of the author (e.g. "joe").
 
 The name of the commit author (e.g. "Joe Random")
 
+=item ${author_via}
+
+The name of the commit author, plus the name of the committer if that is
+different (e.g. "Joe Random" or "Joe Random (via Max Random)")
+
 =item ${branch}
 
 The branch of the commit.
@@ -522,7 +528,7 @@ sub detect_branch_and_module {
 =item shuffle_servers
 
 Returns a shuffled variant of C<< $self-E<gt>servers >>. It considers the last
-successfuly used server by this client instance and puts it first. If there is
+successfully used server by this client instance and puts it first. If there is
 no such server, it considers the state in C<status_dir> and picks the last
 server noted there, if it was used in the last 5 minutes.
 
@@ -757,8 +763,14 @@ returns a colorized string of all commit's changes
 sub colorize_changes {
     my ( $self, $changes ) = @_;
 
+    my @info;
+
     my $changed_files = scalar @$changes;
 
+    my $common_dir = App::KGB::Change->detect_common_dir($changes) // '';
+
+    push @info, $self->colorize( path => "$common_dir/" ) if $common_dir ne '';
+
     if ( $changed_files > $MAGIC_MAX_FILES ) {
         my %dirs;
         for my $c (@$changes) {
@@ -773,14 +785,14 @@ sub colorize_changes {
             ? sprintf( "(%d files in %d dirs)", $changed_files, $dirs )
             : sprintf( "(%d files)",            $changed_files ) );
 
-        return $self->colorize( path => $path_string );
-    }
-    elsif ($changed_files) {
-        return join( ' ', map { $self->colorize_change($_) } @$changes );
+        push @info, $self->colorize( path => $path_string );
     }
     else {
-        return '';
+        push @info, join( ' ', map { $self->colorize_change($_) } @$changes )
+            if @$changes;
     }
+
+    return join( ' ', @info );
 }
 
 =item format_message $template %details
@@ -801,6 +813,23 @@ sub format_message {
     my $result = '';
     warn "# msg = '$msg'" if 0;
 
+    if ($commit) {
+        $p{author_login} = $commit->author if ($commit->author);
+        $p{author_name} = $commit->author_name if ($commit->author_name);
+        $p{author_via} = $commit->author_via if ($commit->author_via);
+        $p{branch} = $commit->branch if ($commit->branch);
+        $p{commit_id} = $commit->id if ($commit->id);
+        $p{log} = $commit->log if ($commit->log);
+    }
+    $p{author_name} ||= $p{author_login};
+    $p{author_login} ||= $p{author_name};
+    $p{author_via} ||= $p{author_name};
+    if (defined($self->module)) {
+        $p{module} = $self->module;
+    } elsif ($commit and defined($commit->module)) {
+        $p{module} = $commit->module;
+    }
+
     while ( $msg =~ /\$\{([^{}]+)?\{([^{}]+)\}([^{}]+)?\}/ps ) {
         my ( $pre, $token, $post ) = ( $1, $2, $3 );
         warn "# pre = '$pre' token = '$token' post = '$post'" if 0;
@@ -813,33 +842,31 @@ sub format_message {
             push @r, project => $self->repo_id;
         }
         elsif ( $token eq 'author-login' ) {
-            push @r, author => $commit
-                ? $commit->author
-                : $p{author_login};
+            push @r, author => $p{author_login};
         }
         elsif ( $token eq 'author-name' ) {
-            push @r, author => $commit
-                ? $commit->author_name
-                : $p{author_name};
+            push @r, author => $p{author_name};
+        }
+        elsif ( $token eq 'author-via' ) {
+            push @r, author => $p{author_via};
         }
         elsif ( $token eq 'branch' ) {
-            push @r, branch => $commit ? $commit->branch : $p{branch};
+            push @r, branch => $p{branch};
         }
         elsif ( $token eq 'module' ) {
-            push @r, module => $self->module
-                // ( $commit ? $commit->module : $p{module} );
+            push @r, module => $p{module};
         }
         elsif ( $token eq 'web-link' ) {
             push @r, web => $p{web_link};
         }
         elsif ( $token eq 'commit' ) {
-            push @r, commit_id => $commit ? $commit->id : $p{commit_id};
+            push @r, commit_id => $p{commit_id};
         }
         elsif ( $token eq 'log' ) {
-            push @r, log => $commit ? $commit->log : $p{log};
+            push @r, log => $p{log};
         }
         elsif ( $token eq 'log-first-line' ) {
-            push @r, log => ( split( /\n/, $commit->log ) )[0];
+            push @r, log => ( split( /\n/, $p{log} ) )[0];
         }
         elsif ( $token eq 'changes' ) {
             push @r, '' => $self->colorize_changes( $commit->changes )
@@ -854,7 +881,11 @@ sub format_message {
         warn "# item = '$item'" if 0;
         next unless defined($item) and $item ne '';
 
-        $result .= $pre // '';
+        if ( defined($pre) ) {
+            # avoid adding multi-spaces or spaces at the beginning
+            $pre =~ s/^\s+// if $result =~ /\s$/ or $result eq '';
+            $result .= $pre;
+        }
         $result .=
             ( $category eq '' )
             ? $item
@@ -1045,7 +1076,8 @@ sub _get_full_user_name {
     return $self->_full_user_name if $self->_full_user_name;
 
     my $user = getpwnam($login);
-    ( my $full_name = $user->gecos ) =~ s/,.*//;
+    my $full_name = $login;
+    ( $full_name = $user->gecos ) =~ s/,.*// if defined $user;
 
     utf8::decode($full_name);
 
